//
//  API.swift
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/20/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

import Foundation

private let kBaseApiUrl_alt         = "https://nameless-meadow-1083.herokuapp.com/"

private let kBaseApiUrl             = "https://polar-tundra-5280.herokuapp.com/api/"

private let kReadMessagesApiMethod  = "read_messages"
private let kSendMessageApiMethod   = "send_message"
private let kRegisterUserApiMethod  = "register_user"

private let timeout = 20.0

private let kUserKey = "to"
private let kFromKey = "from"

private let kLastMessageId  = "mid"

private let kFingerprintKey = "fingerprint"
private let kTokenKey       = "token"

private let kMessageDataKey = "data"

private let kRequestTypeKey = "request_type"
private let kRequestDataKey = "request_data"

private let IS_ALT = false

@objc public class API:NSObject {
    
    var dataStore:DataStore
    var keyStore:SecKeyWrapper
    
    init(dataStore:DataStore, keyStore:SecKeyWrapper) {
        self.dataStore = dataStore
        self.keyStore = keyStore
    }

    // MARK: Internal API helpers
    private func doPOST(httpBodyData:NSData, url:NSURL, completionHandler:((NSData?, NSError?) -> Void)) {
        
        var request:NSMutableURLRequest = NSMutableURLRequest(URL: url)
        request.timeoutInterval = timeout
        request.cachePolicy = NSURLRequestCachePolicy.ReturnCacheDataElseLoad
        request.HTTPMethod = "POST"
        request.addValue("application/json", forHTTPHeaderField: "Content-Type")
        request.HTTPBody = httpBodyData
        
        NSURLConnection.sendAsynchronousRequest(request, queue: NSOperationQueue()) { (_, data, error) -> Void in
            
            if let theData = data {
                println(NSString(data: theData, encoding: NSUTF8StringEncoding))
            }
            
            completionHandler(data, error)
        }
    }
    
    private func parseMessages(data:NSData?) -> NSArray? {
        
        if data == nil {
            return []
        }
        
        var err:NSError?
        
        var jsonResultObj: AnyObject? = NSJSONSerialization.JSONObjectWithData(data!, options: NSJSONReadingOptions.MutableContainers, error: &err)
        
        if err != nil {
            println(err)
            return nil
        }
        
        var jsonResult:NSDictionary
        if let theJsonResult:NSDictionary = jsonResultObj as? NSDictionary {
            jsonResult = theJsonResult
        } else {
            return nil
        }
        
        let userMessageList:NSMutableArray = NSMutableArray()
        
        let jsonList:NSArray = jsonResult.objectForKey(kMessageDataKey) as NSArray
        
        for userMessageDictionary in jsonList {
            if let userMessage = UserMessage.deserializeDictionary(userMessageDictionary as NSDictionary, dataStore: self.dataStore, keyStore: self.keyStore, transported: true) {
                userMessageList.addObject(userMessage)
            }
        }
        
        return userMessageList
    }
    
    
    // MARK: API Methods
    public func registerUser(fingerprint:String, token:String, completionHandler:((Bool, NSError?) -> Void)) {
        
        let data = NSJSONSerialization.dataWithJSONObject([kFingerprintKey: fingerprint, kTokenKey: token], options: NSJSONWritingOptions.PrettyPrinted, error: nil)

        self.doPOST(data!, url: NSURL(string:kBaseApiUrl + kRegisterUserApiMethod)!, { (data:NSData?, error:NSError?) -> Void in
            if error != nil {
                completionHandler(false, error)
                return
            }
            
            completionHandler(true, nil)
        })
    }
    
    public func getNewMessages(to:String, mid:NSNumber?, completionHandler:((NSArray?, NSError?) -> Void)) {
        
        var data:NSData?
        var url:NSURL
        
        if (IS_ALT) {
            var params:[String:AnyObject] = [kRequestTypeKey: kReadMessagesApiMethod, kRequestDataKey: [kUserKey: to]]
            
            if let messageId = mid {
                params[kLastMessageId] = messageId
            }
            
            data = NSJSONSerialization.dataWithJSONObject(params, options: NSJSONWritingOptions.PrettyPrinted, error: nil)
            let dataStr = NSString(data: data!, encoding: NSUTF8StringEncoding)
            url = NSURL(string:kBaseApiUrl_alt)!
            
        } else {
            var params:[String:AnyObject] = [kUserKey: to]
            
            if let messageId = mid {
                params[kLastMessageId] = messageId
            }
            
            data = NSJSONSerialization.dataWithJSONObject(params, options: NSJSONWritingOptions.PrettyPrinted, error: nil)
            url = NSURL(string:kBaseApiUrl + kReadMessagesApiMethod)!
        }
        
        self.doPOST(data!, url: url, { (data:NSData?, error:NSError?) -> Void in
            if error != nil {
                completionHandler(nil, error)
                return
            }
            
            completionHandler(self.parseMessages(data), nil)
        })
    }
    
    public func getNewMessages(to:String, from:String, mid:NSNumber?, completionHandler:((NSArray?, NSError?) -> Void)) {
        
        var data:NSData?
        var url:NSURL
        
        if (IS_ALT) {
            var params:[String:AnyObject]  = [kRequestTypeKey: kReadMessagesApiMethod, kRequestDataKey: [kUserKey: to, kFromKey: from]]
            
            if let messageId = mid {
                params[kLastMessageId] = messageId
            }
            
            data = NSJSONSerialization.dataWithJSONObject(params, options: NSJSONWritingOptions.PrettyPrinted, error: nil)
            url = NSURL(string:kBaseApiUrl_alt)!
            
        } else {
            var params:[String:AnyObject] = [kUserKey: to, kFromKey: from]
            
            if let messageId = mid {
                params[kLastMessageId] = messageId
            }
            
            data = NSJSONSerialization.dataWithJSONObject(params, options: NSJSONWritingOptions.PrettyPrinted, error: nil)
            url = NSURL(string:kBaseApiUrl + kReadMessagesApiMethod)!
        }
        

        self.doPOST(data!, url: url, { (data:NSData?, error:NSError?) -> Void in
            if error != nil {
                completionHandler(nil, error)
                return
            }
            
            completionHandler(self.parseMessages(data), nil)
        })
    }
    
    public func sendMessage(userMessage:UserMessage, completionHandler:((Bool, NSError?) -> Void)) {
        
        var message:NSDictionary = userMessage.serializeToDictionary(self.dataStore, keyStore: self.keyStore, transportable: true) as NSDictionary
        
        var data:NSData?
        var url:NSURL
        
        if (IS_ALT) {
            let params = [kRequestTypeKey: kSendMessageApiMethod, kRequestDataKey: message]
            
            var err:NSError?
            data = NSJSONSerialization.dataWithJSONObject(params, options: NSJSONWritingOptions.PrettyPrinted, error: &err)
            url = NSURL(string:kBaseApiUrl_alt)!

        } else {
            data = NSJSONSerialization.dataWithJSONObject(message, options: NSJSONWritingOptions.PrettyPrinted, error: nil)
            url = NSURL(string:kBaseApiUrl + kSendMessageApiMethod)!
        }

        self.doPOST(data!, url: url, { (data:NSData?, error:NSError?) -> Void in
            if error != nil {
                completionHandler(false, error)
                return
            }
            
            completionHandler(true, nil)
        })
    }
    
    
}