//
//  ContactsViewController.m
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/17/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

#import "ContactsViewController.h"
#import "UIColor+FlatColors.h"
#import "NSString+Additions.h"
#import "ConversationsViewController.h"

@implementation ContactCell
- (void)awakeFromNib {
    
    self.contactView.backgroundColor = [UIColor flatTurquoiseColor];
    self.contactView.layer.cornerRadius = self.contactView.frame.size.width/2;
    
    self.initialsLabel = [[UILabel alloc] initWithFrame: self.contactView.bounds];
    self.initialsLabel.backgroundColor = [UIColor clearColor];
    self.initialsLabel.font = [UIFont fontWithName: @"HelveticaNeue-Light" size: 25];
    self.initialsLabel.textColor = [UIColor whiteColor];
    self.initialsLabel.textAlignment = NSTextAlignmentCenter;
    
    self.fingerprintLabel.textColor = [UIColor flatTurquoiseColor];
    
    [self.contactView addSubview: self.initialsLabel];
    
}

- (void) setUsername: (NSString *) username {
    self.usernameLabel.text = username;
    
    if (username.length == 0) {
        self.initialsLabel.text = @"?";
        return;
    }
    
    NSArray *components = [username componentsSeparatedByString: @" "];
    if (components.count == 1) {
        self.initialsLabel.text = [[username substringWithRange: NSMakeRange(0, 1)] uppercaseString];
        return;
    }
    
    NSString *firstName = components[0];
    NSString *lastName = components[1];

    self.initialsLabel.text = [[NSString stringWithFormat: @"%@%@", [firstName substringToIndex:1],[lastName substringToIndex:1]] uppercaseString];
}

- (void) setFingerprint: (NSString *) fingerprint {
    self.fingerprintLabel.text = [NSString addSpacesToFingerprint: fingerprint];
}
@end


@interface ContactsViewController()<DZNEmptyDataSetSource, DZNEmptyDataSetDelegate>

@property (nonatomic, strong) SecKeyWrapper *keyStore;
@property (nonatomic, strong) DataStore *dataStore;
@property (nonatomic, strong) ConversationManager *conversationManager;
@property (nonatomic, strong) NSArray *contactList;
@property (nonatomic, strong) User *me;
@end

@implementation ContactsViewController

- (void) viewDidLoad {
    [super viewDidLoad];
    
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Close" style: UIBarButtonItemStyleDone target: self action: @selector(closeContacts)];

    
    self.navigationController.navigationBar.tintColor = [UIColor whiteColor];
    self.navigationController.navigationBar.barTintColor = [UIColor flatTurquoiseColor];
    NSDictionary *navbarTitleTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
                                               [UIColor whiteColor],NSForegroundColorAttributeName,nil];
    
    self.navigationController.navigationBar.titleTextAttributes = navbarTitleTextAttributes;
    
    self.tableView.tableFooterView = [[UIView alloc] init];
    self.title = @"Contacts";
    
    self.tableView.emptyDataSetDelegate = self;
    self.tableView.emptyDataSetSource = self;

    //reference app objects
    self.keyStore = [SecKeyWrapper sharedWrapper];
    self.dataStore = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).dataStore;
    self.conversationManager = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).conversationManager;

    NSString *publicKeyString = [[self.keyStore getPublicKeyBits] base64EncodedStringWithOptions: NSDataBase64Encoding64CharacterLineLength];
    NSString *username = [self.dataStore myUsername];
    self.me = [[User alloc] initWithUsername: username publicKey: publicKeyString];

    NSMutableArray *contacts = [[NSMutableArray alloc] initWithArray: [self.dataStore friendList].allValues];
    [contacts addObject: self.me];
    self.contactList = [NSArray arrayWithArray: contacts];
}

- (void) closeContacts {
    [self.tableView reloadData];
    [self dismissViewControllerAnimated: YES completion: nil];
}
#pragma mark - table view delegate
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.contactList.count;
}
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    ContactCell *cell = [tableView dequeueReusableCellWithIdentifier:@"ContactCell"];
    
    [cell setUsername: ((User *)[self.contactList objectAtIndex: indexPath.row]).username];
    [cell setFingerprint: [[((User *)[self.contactList objectAtIndex: indexPath.row]) fingerprint] uppercaseString]];
    cell.selectionStyle = UITableViewCellSelectionStyleBlue;
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    User *selectedUser = (User *)[self.contactList objectAtIndex: indexPath.row];
    
    if (selectedUser == self.me) {
        return;
    }
    
    NSArray *conversationList = [self.conversationManager allConversations];
    
    Conversation *chosenConversation = nil;
    NSString *selectedUserFingerprint = [selectedUser fingerprint];
    for (Conversation *conversation in conversationList) {
        if ([conversation.recipient isEqualToString: selectedUserFingerprint]) {
            chosenConversation = conversation;
            break;
        }
    }
    
    if (chosenConversation == nil) {
        chosenConversation = [[Conversation alloc] initWithRecipient: selectedUserFingerprint messages: nil];
    }
    
    [self dismissViewControllerAnimated: YES completion:^{
        [self.delegate choseConversation: chosenConversation];
    }];
    
}

-(BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    User *user = (User *)[self.contactList objectAtIndex: indexPath.row];
    
    return [user.fingerprint isEqualToString: self.me.fingerprint] == NO;
}

// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        User *user = (User *)[self.contactList objectAtIndex: indexPath.row];
        [self.dataStore deleteUserForFingerprint: user.fingerprint];
        [self.conversationManager deleteConversationForRecipient: user.fingerprint];
        
        
        NSMutableArray *contacts = [[NSMutableArray alloc] initWithArray: [self.dataStore friendList].allValues];
        [contacts addObject: self.me];
        self.contactList = [NSArray arrayWithArray: contacts];
        
        [self.tableView reloadData];
        
        if (self.delegate) {
            [self.delegate deletedUser];
        }
    }   
}


- (NSAttributedString *)titleForEmptyDataSet:(UIScrollView *)scrollView {
    
    NSString *text = @"No Contacts Yet";
    
    NSDictionary *attributes = @{NSFontAttributeName: [UIFont boldSystemFontOfSize:18.0],
                                 NSForegroundColorAttributeName: [UIColor darkGrayColor]};
    
    return [[NSAttributedString alloc] initWithString:text attributes:attributes];
}

- (NSAttributedString *)descriptionForEmptyDataSet:(UIScrollView *)scrollView {
    
    NSString *text = @"Exchange keys with friend and then securely message them anytime!";
    
    NSMutableParagraphStyle *paragraph = [NSMutableParagraphStyle new];
    paragraph.lineBreakMode = NSLineBreakByWordWrapping;
    paragraph.alignment = NSTextAlignmentCenter;
    
    NSDictionary *attributes = @{NSFontAttributeName: [UIFont systemFontOfSize:14.0],
                                 NSForegroundColorAttributeName: [UIColor lightGrayColor],
                                 NSParagraphStyleAttributeName: paragraph};
    
    return [[NSAttributedString alloc] initWithString:text attributes:attributes];
}

- (UIImage *)imageForEmptyDataSet:(UIScrollView *)scrollView {
    
    return [UIImage imageNamed:@"ph_contacts"];
}

- (BOOL)emptyDataSetShouldDisplay:(UIScrollView *)scrollView {
    
    return self.contactList.count == 0;
}

@end
