//
//  Conversation.swift
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/19/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

import Foundation

private let kRecipientKey = "recipient"
private let kMessagesKey = "messages"
private let kHasNewMessageKey = "has_new"

@objc public class Conversation:NSObject {
    
    var recipient:String
    var lastUpdated:NSDate?
    var lastMessageId:NSNumber?

    private var messages:NSMutableArray
    private var digestDict:[String:Bool]
    
    public var hasNewMessage:Bool
    
    init(recipient:String, messages:NSArray?) {
        self.recipient = recipient
        self.messages = NSMutableArray()
        self.digestDict = [:]
        
        if let theMessages = messages {
            for aMessage in theMessages {
                self.messages.addObject(aMessage)
                
                let digestStr:String = (aMessage as UserMessage).digest.base64EncodedStringWithOptions(NSDataBase64EncodingOptions.Encoding64CharacterLineLength)

                self.digestDict[digestStr] = true
            }
            self.lastUpdated = (self.messages.lastObject! as UserMessage).message.date
            self.lastMessageId = (self.messages.lastObject! as UserMessage).messageId
        }
        
        self.hasNewMessage = false;
    }
    
    public func allMessages() -> NSArray {
        return NSArray(array: self.messages)
    }
    
    public func mostRecentMessage() -> UserMessage? {
        return self.messages.lastObject as? UserMessage
    }
    
    public func addMessage(message:UserMessage) -> Bool{

        let digestStr:String = message.digest.base64EncodedStringWithOptions(NSDataBase64EncodingOptions.Encoding64CharacterLineLength)
        
        
        self.updateLastMessageId(message.messageId)

        //check that we don't already have this message
        if let digestExists = self.digestDict[digestStr] {
            return false
        }
        
        self.messages.addObject(message)
        self.digestDict[digestStr] = true
        self.lastUpdated = message.message.date
        
        return true
    }
    
    public func updateLastMessageId(messageId:NSNumber?) {
        if messageId == nil {
            return
        }
        
        if let currentMessageId = self.lastMessageId {
            if (currentMessageId.compare(messageId!) == NSComparisonResult.OrderedAscending) {
                self.lastMessageId = messageId
            }
        }
        else  {
            self.lastMessageId = messageId
        }
    }
    
    public func serialize(dataStore:DataStore, keyStore:SecKeyWrapper) -> NSString?{
        let parameters = NSMutableDictionary()
        parameters.setObject(self.recipient, forKey:  kRecipientKey)
        parameters.setObject(NSNumber(bool: self.hasNewMessage), forKey: kHasNewMessageKey)
        
        let serializedMessages:NSMutableArray = NSMutableArray()
        
        for message in self.messages {
            let userMessage:UserMessage  = message as UserMessage
            
            if let serializedUserMessage = userMessage.serialize(dataStore, keyStore: keyStore, transportable: false) {
                serializedMessages.addObject(serializedUserMessage)
            }
            
        }
        
        parameters.setObject(serializedMessages, forKey: kMessagesKey)
        
        var err:NSError?
        
        let jsonData = NSJSONSerialization.dataWithJSONObject(parameters, options: NSJSONWritingOptions.PrettyPrinted, error:  &err)
        
        if (err != nil) {
            println(err)
            return nil
        }
        
        return NSString(data: jsonData!, encoding: NSUTF8StringEncoding)
    }
    
    public class func deserialize(object:NSString, dataStore:DataStore, keyStore:SecKeyWrapper) -> Conversation? {
        let data = object.dataUsingEncoding(NSUTF8StringEncoding)
        
        var err:NSError?
        
        var jsonResult: NSDictionary = NSJSONSerialization.JSONObjectWithData(data!, options: NSJSONReadingOptions.MutableContainers, error: &err) as NSDictionary
        
        if err != nil {
            println(err)
            return nil
        }
        
        
        let recipient  = jsonResult.objectForKey(kRecipientKey) as NSString?
        let hasNew     = jsonResult.objectForKey(kHasNewMessageKey) as NSNumber?

        let messageList:NSMutableArray = NSMutableArray()
        
        if let serializedMessages = jsonResult.objectForKey(kMessagesKey) as NSArray? {
            
            for msg in serializedMessages {
                if let userMessage = UserMessage.deserializeString(msg as NSString, dataStore: dataStore, keyStore: keyStore, transported: false) {
                    
                    messageList.addObject(userMessage)
                }

            }
        }
        
        let createdConvo = Conversation(recipient: recipient!, messages: messageList)
        
        if let isNewMessage = hasNew {
            createdConvo.hasNewMessage = isNewMessage.boolValue
        }
        
        return createdConvo
    }
    
    
    
    
    public class func allConversations(dataStore:DataStore, keyStore:SecKeyWrapper) -> NSArray {
        let conversationList:NSMutableArray = NSMutableArray()
        
        return conversationList
    }
}