//
//  ConversationManager.swift
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/19/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

import Foundation


private let kConversationsKey = "conversation-dict"
@objc public class ConversationManager:NSObject {
    
    var dataStore:DataStore
    var keyStore:SecKeyWrapper
    
    private var conversations:NSMutableDictionary
    
    public var lastMessageId:NSNumber?
    
    init(dataStore:DataStore, keyStore:SecKeyWrapper) {
        self.dataStore = dataStore
        self.keyStore = keyStore
        self.conversations = NSMutableDictionary()
        
        super.init()

        self.initConversations()
    }
    
    private func initConversations() {
        if let savedConversations = NSUserDefaults.standardUserDefaults().dictionaryForKey(kConversationsKey) {
            
            for recip in savedConversations.keys {
                let converationObject = savedConversations[recip] as NSString
                if let convo:Conversation = Conversation.deserialize(converationObject as NSString, dataStore: self.dataStore, keyStore: self.keyStore) {
                    self.conversations.setObject(convo, forKey: recip as NSString);
                    
                    //set the last message id
                    if let lastMessageId = convo.lastMessageId {
                        self.updateLastMessageId(lastMessageId)
                    }
                }
            }
        }
    }
    
    private func updateLastMessageId(newMessageId:NSNumber) {
        if self.lastMessageId == nil {
            self.lastMessageId = newMessageId
        }
        
        if newMessageId.compare(self.lastMessageId!) == NSComparisonResult.OrderedDescending {
            self.lastMessageId = newMessageId
        }
    }
    
    public func addConversation(conversation:Conversation) {
        self.conversations.setObject(conversation, forKey: conversation.recipient)
    }
    
    public func getConversation(recipient:String) -> Conversation? {
        return self.conversations.objectForKey(recipient) as? Conversation
    }
    
    public func allConversations() -> NSArray {
        let sortedConversations = sorted(self.conversations.allValues, {(ca:AnyObject, cb:AnyObject) -> Bool in
                return (ca as Conversation).lastUpdated!.timeIntervalSince1970 > (cb as Conversation).lastUpdated!.timeIntervalSince1970
            })
        
        return NSArray(array: sortedConversations)
    }
    
    public func addMessage(message:UserMessage, recipient:String) -> Conversation?{
        
        
        if let conversation:Conversation = self.conversations.objectForKey(recipient) as? Conversation {
            
            let isNew = conversation.addMessage(message) && (recipient == message.from)
            conversation.hasNewMessage = isNew
            
            //set the last message id
            if let lastMessageId = conversation.lastMessageId {
                self.updateLastMessageId(lastMessageId)
            }
            
            return nil
        }
        
        
        var newConversation:Conversation = Conversation(recipient: recipient, messages: NSArray(object: message))
        newConversation.hasNewMessage = true
        self.conversations.setObject(newConversation, forKey: recipient)

        //set the last message id
        if let lastMessageId = newConversation.lastMessageId {
            self.updateLastMessageId(lastMessageId)
        }

        return newConversation
    }
    
    public func deleteConversationForRecipient(recipient:String) {
        self.conversations.removeObjectForKey(recipient)
    }
        
    public func deleteAllData() {
        self.conversations = NSMutableDictionary()
        NSUserDefaults.standardUserDefaults().removeObjectForKey(kConversationsKey)
        NSUserDefaults.standardUserDefaults().synchronize()
    }
    
    public func save() {
        let serializedDictionary:NSMutableDictionary = NSMutableDictionary()

        for recipient in self.conversations.allKeys {
            let conversation:Conversation = self.conversations.objectForKey(recipient) as Conversation
            
            if let serializedConversation = conversation.serialize(self.dataStore, keyStore: self.keyStore) {
                serializedDictionary.setObject(serializedConversation, forKey: recipient as String)
            }
        }
        
        NSUserDefaults.standardUserDefaults().setObject(serializedDictionary, forKey: kConversationsKey)
        NSUserDefaults.standardUserDefaults().synchronize()
    }
}