//
//  MessagesViewController.m
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/14/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

#import "ConversationsViewController.h"
#import "AppDelegate.h"
#import "UIScrollView+EmptyDataSet.h"
#import "MessageViewController.h"
#import "NSString+Additions.h"
#import "RTSpinKitView.h"
#import "UIColor+FlatColors.h"

#define kUpdateConversationNotification @"update_conversation_notification"

@interface ConversationsViewController()<DZNEmptyDataSetDelegate, DZNEmptyDataSetSource> {
    BOOL firstLoadAttempted;
}

@property (nonatomic, strong) SecKeyWrapper *keyStore;
@property (nonatomic, strong) DataStore *dataStore;
@property (nonatomic, strong) ConversationManager *conversationManager;
@property (nonatomic, strong) API *api;
@property (nonatomic, strong) NSString *myFingerprint;
@property (nonatomic, strong) NSTimer *refreshTimer;
@property (nonatomic, strong) NSArray *conversationList;

@property (nonatomic, strong) MessageViewController *openMessageViewController;
@end

@implementation ConversationsViewController

- (void) viewDidLoad {
    [super viewDidLoad];
    
    self.keyStore = [SecKeyWrapper sharedWrapper];
    self.dataStore = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).dataStore;
    self.conversationManager = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).conversationManager;
    self.api = [[API alloc] initWithDataStore: self.dataStore keyStore: self.keyStore];
    
    self.tableView.tableFooterView = [[UIView alloc] init];
    
    self.tableView.emptyDataSetSource = self;
    self.tableView.emptyDataSetDelegate = self;
    
    self.conversationList = [self.conversationManager allConversations];

    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(receivedNewPushMessage) name: kAppPushNotificationNewMessage object: nil];

}

- (void) viewWillAppear:(BOOL)animated {
    [super viewWillAppear: animated];
    [self.tabBarController.tabBar setHidden: NO];
    
    self.openMessageViewController = nil;

    self.myFingerprint = [NSString getFingerprintFromPublicKey: [self.keyStore getPublicKeyBits]];
    self.conversationList = [self.conversationManager allConversations];
    [self.tableView reloadData];

    
    if (!self.refreshTimer) {
        self.refreshTimer = [NSTimer scheduledTimerWithTimeInterval: 30 target: self selector: @selector(loadMessages) userInfo: nil repeats: YES];
        [self loadMessages];
    }
    
}

- (void) viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear: animated];
}

- (void)dealloc {
    if (self.refreshTimer) {
        [self.refreshTimer invalidate];
        self.refreshTimer = nil;
    }
}

- (void)openConversation:(Conversation *)conversation {
    MessageViewController *messageViewController = [self.storyboard instantiateViewControllerWithIdentifier: @"MessageViewController"];
    messageViewController.conversation = conversation;
    
    self.openMessageViewController = messageViewController;
    
    [self.navigationController pushViewController: messageViewController animated: YES];
    [self.tabBarController.tabBar setHidden: YES];
}

#pragma mark - Push

- (void) receivedNewPushMessage {
    [self loadMessages];
}

#pragma mark - Request Messages from Server

- (void) loadMessages {
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    
    [self.api getNewMessages: self.myFingerprint mid: self.conversationManager.lastMessageId completionHandler:
     ^(NSArray *messages, NSError *error) {
         [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
         
         firstLoadAttempted = YES;
         if (error) {
             NSLog(@"Error getting new messages %@", error);
             return;
         }
         

         for (UserMessage *userMessage in messages) {
             if (userMessage.message.isRevokeMessage) {
                 //TODO:handle revoke
             }
             [self.conversationManager addMessage: userMessage recipient: userMessage.from];
         }
         
         self.conversationList = [self.conversationManager allConversations];
         dispatch_async(dispatch_get_main_queue(), ^{
             [self.tableView reloadData];
         });
         
         if (self.openMessageViewController) {
             [self.openMessageViewController loadNewMessages];
         }
    }];
}

#pragma mark - table view delegate
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.conversationList.count;
}
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 70;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    ConversationCell *cell = [tableView dequeueReusableCellWithIdentifier:@"ConversationCell"];
    cell.separatorInset = UIEdgeInsetsZero;
    Conversation *conversation = (Conversation *)[self.conversationList objectAtIndex: indexPath.row];
    
    if ([conversation hasNewMessage]) {
        cell.backgroundColor = [[UIColor flatEmeraldColor] colorWithAlphaComponent: 0.2f];
    } else {
        cell.backgroundColor = [UIColor whiteColor];
    }
    
    User *recipient = [[self.dataStore friendList] objectForKey: conversation.recipient];
    [cell setUsername: recipient.username];
    
    ProtocolMessage *message = [conversation mostRecentMessage].message;
    cell.messageLabel.text = message.content;

    
    NSDate *theMessageDate = [conversation mostRecentMessage].message.date;
    
    
    NSDateComponents *messageComp = [[NSCalendar currentCalendar] components:NSCalendarUnitEra|NSCalendarUnitYear|NSCalendarUnitDay|NSCalendarUnitWeekday fromDate: theMessageDate];
   
    NSDateComponents *todayComp = [[NSCalendar currentCalendar] components:NSCalendarUnitEra|NSCalendarUnitYear|NSCalendarUnitDay|NSCalendarUnitWeekday fromDate:[NSDate date]];

    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];

    if([todayComp day] == [messageComp day]
       && [todayComp month] == [todayComp month]
       && [todayComp year] == [todayComp year]) {
        [dateFormatter setDateFormat: @"h:mm a"];
        cell.dateLabel.text = [dateFormatter stringFromDate: theMessageDate];
    }
    else if([todayComp day] - 1 == [messageComp day]
       && [todayComp month] == [todayComp month]
       && [todayComp year] == [todayComp year]) {
        cell.dateLabel.text = @"Yesterday";
    }
    else if([todayComp weekOfMonth] == [messageComp weekOfMonth]
            && [todayComp month] == [todayComp month]
            && [todayComp year] == [todayComp year]) {
        [dateFormatter setDateFormat: @"EEEE"];
        cell.dateLabel.text = [dateFormatter stringFromDate: theMessageDate];
    }
    else {
        [dateFormatter setDateFormat: @"M/d"];
        cell.dateLabel.text = [dateFormatter stringFromDate: theMessageDate];
    }
    
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    Conversation *conversation = [self.conversationList objectAtIndex: indexPath.row];
    conversation.hasNewMessage = NO;
    [self openConversation: conversation];
}

// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        Conversation *conversation = (Conversation *)[self.conversationList objectAtIndex: indexPath.row];
        [self.conversationManager deleteConversationForRecipient: conversation.recipient];
        
        self.conversationList = [self.conversationManager allConversations];

        [self.tableView reloadData];
    }
}

- (NSAttributedString *)titleForEmptyDataSet:(UIScrollView *)scrollView {
    
    if (firstLoadAttempted == NO) {
        return [[NSAttributedString alloc] initWithString: @"" attributes: @{}];
    }
    
    NSString *text = @"No Conversations Yet :(";
    
    NSDictionary *attributes = @{NSFontAttributeName: [UIFont boldSystemFontOfSize:18.0],
                                 NSForegroundColorAttributeName: [UIColor darkGrayColor]};
    
    return [[NSAttributedString alloc] initWithString:text attributes:attributes];
}

- (NSAttributedString *)descriptionForEmptyDataSet:(UIScrollView *)scrollView {
    
    if (firstLoadAttempted == NO) {
        return [[NSAttributedString alloc] initWithString: @"" attributes: @{}];
    }
    
    NSString *text = @"Tap the 'Exchange' tab, to get a friends key. Then message them securely!";
    
    NSMutableParagraphStyle *paragraph = [NSMutableParagraphStyle new];
    paragraph.lineBreakMode = NSLineBreakByWordWrapping;
    paragraph.alignment = NSTextAlignmentCenter;
    
    NSDictionary *attributes = @{NSFontAttributeName: [UIFont systemFontOfSize:14.0],
                                 NSForegroundColorAttributeName: [UIColor lightGrayColor],
                                 NSParagraphStyleAttributeName: paragraph};
    
    return [[NSAttributedString alloc] initWithString:text attributes:attributes];
}

- (UIImage *)imageForEmptyDataSet:(UIScrollView *)scrollView {
    
    if (firstLoadAttempted == NO) {
        return nil;
    }
    
    return [UIImage imageNamed:@"ph_messages"];
}

- (UIView *)customViewForEmptyDataSet:(UIScrollView *)scrollView {
    if (firstLoadAttempted) {
        return nil;
    }
    
    RTSpinKitView *spinView = [[RTSpinKitView alloc] initWithStyle: RTSpinKitViewStylePulse color: [UIColor flatPeterRiverColor]];
    spinView.spinnerSize = 200;
    spinView.frame = CGRectMake(spinView.frame.origin.x, spinView.frame.origin.y-100, 200, 200);
    return spinView;
}

- (BOOL)emptyDataSetShouldDisplay:(UIScrollView *)scrollView {
    
    return self.conversationList.count == 0;
}

@end

@implementation ConversationCell

- (void)awakeFromNib {
    self.contactView.backgroundColor = [UIColor flatTurquoiseColor];
    self.contactView.layer.cornerRadius = self.contactView.frame.size.width/2;

    self.initialsLabel = [[UILabel alloc] initWithFrame: self.contactView.bounds];
    self.initialsLabel.backgroundColor = [UIColor clearColor];
    self.initialsLabel.font = [UIFont fontWithName: @"HelveticaNeue-Light" size: 25];
    self.initialsLabel.textColor = [UIColor whiteColor];
    self.initialsLabel.textAlignment = NSTextAlignmentCenter;
    self.initialsLabel.highlightedTextColor = [UIColor flatPeterRiverColor];

    [self.contactView addSubview: self.initialsLabel];
}

- (void) setUsername: (NSString *) username {
    self.titleLabel.text = username;
    
    if (username.length == 0) {
        self.initialsLabel.text = @"?";
        return;
    }
    
    NSArray *components = [username componentsSeparatedByString: @" "];
    if (components.count == 1) {
        self.initialsLabel.text = [[username substringWithRange: NSMakeRange(0, 1)] uppercaseString];
        return;
    }
    
    NSString *firstName = components[0];
    NSString *lastName = components[1];
    
    self.initialsLabel.text = [[NSString stringWithFormat: @"%@%@", [firstName substringToIndex:1],[lastName substringToIndex:1]] uppercaseString];
}

@end