//
//  MessageViewController.m
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/27/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

#import "MessageViewController.h"
#import "AppDelegate.h"
#import <WebKit/WebKit.h>
#import "UserViewController.h"

#define kJsTemplate @"insertMessage('%@', '%@', '%@', %@);"

@interface MessageViewController()<UITextFieldDelegate, UIWebViewDelegate> {
    BOOL isPageLoaded;
}
@property (nonatomic, strong) User *recipient;
@property (nonatomic, strong) User *me;

@property (nonatomic, strong) SecKeyWrapper *keyStore;
@property (nonatomic, strong) DataStore *dataStore;
@property (nonatomic, strong) ConversationManager *conversationManager;
@property (nonatomic, strong) API *api;

@property (nonatomic, strong) IBOutlet UITableViewCell *mainCell;
@property (nonatomic, strong) IBOutlet UIWebView *webView;
@property (nonatomic, strong) IBOutlet UITextField *messageField;
@property (nonatomic, strong) IBOutlet UIBarButtonItem *sendButton;
@property (nonatomic, strong) IBOutlet UIToolbar *toolbar;
@property (nonatomic, strong) NSMutableDictionary *addedMessages;

@end
@implementation MessageViewController

- (void) viewDidLoad {
    [super viewDidLoad];
    
    self.tableView.scrollEnabled = NO;
    
    //reference app objects
    self.keyStore = [SecKeyWrapper sharedWrapper];
    self.dataStore = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).dataStore;
    self.conversationManager = ((AppDelegate *)[[UIApplication sharedApplication] delegate]).conversationManager;
    self.api = [[API alloc] initWithDataStore: self.dataStore keyStore: self.keyStore];

    self.addedMessages = [[NSMutableDictionary alloc] init];
    NSString *publicKeyString = [[self.keyStore getPublicKeyBits] base64EncodedStringWithOptions: NSDataBase64Encoding64CharacterLineLength];
    NSString *username = [self.dataStore myUsername];
    self.me = [[User alloc] initWithUsername: username publicKey: publicKeyString];
    
    NSMutableArray *contacts = [[NSMutableArray alloc] initWithArray: [self.dataStore friendList].allValues];
    [contacts addObject: self.me];
    
    if ([self.conversation.recipient isEqual: [self.me fingerprint]]) {
        self.recipient = self.me;
    } else {
        self.recipient = [[self.dataStore friendList] objectForKey: self.conversation.recipient];
    }
    
    self.messageField.delegate = self;
    self.messageField.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    self.title = self.recipient.username;
    
    //top bar
    
    UIButton *infoButton = [UIButton buttonWithType: UIButtonTypeDetailDisclosure];
    infoButton.tintColor = [UIColor whiteColor];
    [infoButton addTarget: self action: @selector(showUserDetails) forControlEvents: UIControlEventTouchUpInside];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView: infoButton];
    
    
    //webview stuff
    self.webView.delegate = self;
    self.webView.scrollView.showsHorizontalScrollIndicator = NO;
    self.webView.scrollView.showsVerticalScrollIndicator = NO;
    self.webView.scrollView.keyboardDismissMode =UIScrollViewKeyboardDismissModeOnDrag;
    self.webView.backgroundColor = [UIColor whiteColor];
    
    [self updateMessages];
}


- (void) viewWillAppear:(BOOL)animated {
    [super viewWillAppear: animated];
    
    [self loadNewMessages];
}

- (void) viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear: animated];
}

#pragma mark - User Detes

- (void) showUserDetails {
    UINavigationController *userNav = [self.storyboard instantiateViewControllerWithIdentifier: @"UserViewNav"];
    
    UserViewController *userViewController = [userNav.viewControllers objectAtIndex: 0];
    userViewController.viewMode = FriendViewMode;
    userViewController.user = self.recipient;
    
    [self presentViewController: userNav animated: YES completion: nil];
}

#pragma mark - Requesting messages
- (void) loadNewMessages  {
    if (isPageLoaded == NO){
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        for (UserMessage *message in [self.conversation allMessages]) {
            [self displayUserMessage: message];
        }
    });
}

- (void) updateMessages {
    NSString *messagesFile = [[NSBundle mainBundle] pathForResource: @"messages" ofType:@"html"];
    NSString *html = [NSString stringWithContentsOfFile: messagesFile encoding: NSUTF8StringEncoding error: nil];
    
    [self.webView loadHTMLString: html baseURL: [NSURL fileURLWithPath: messagesFile]];
}



- (IBAction) sendTapped {
    NSString *messageText = self.messageField.text;
    NSString *recipientFingerprint = [self.recipient fingerprint];
    
    ProtocolMessage *textMessage = [[ProtocolMessage alloc] initWithContent: messageText date: [NSDate date]];
    UserMessage *newUserMessage = [[UserMessage alloc] initTo: recipientFingerprint from:[self.me fingerprint] message: textMessage signature: nil digest: nil messageId: nil];
    
    //send the message to the server:
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    [self.api sendMessage: newUserMessage completionHandler:^(BOOL sent, NSError *error) {
        [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
        
        if (error) {
            NSLog(@"%@", error);
            //TODO:Show error
            return;
        }
        
        if (sent) {
            NSLog(@"Sent!");
            Conversation *newConversation = [self.conversationManager addMessage: newUserMessage recipient: recipientFingerprint];
            
            if (newConversation != nil) {
                self.conversation = newConversation;
            }
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [self displayUserMessage: newUserMessage];
            });
        }
    }];
    
    self.messageField.text = @"";
}

- (void) displayUserMessage: (UserMessage *) message {
    
    if ([self.addedMessages objectForKey: message.digest] != nil) {
        return;
    }
    
    ProtocolMessage *textMessage = message.message;
    
    NSString *messageText = [self htmlEncode: textMessage.content];
    NSString *jsonDate = [NSString stringWithFormat:@"%f", [textMessage.date timeIntervalSince1970]];
    
    BOOL isRecieved = ([message.to isEqualToString: [self.me fingerprint]] && !([message.to isEqualToString: message.from]));
    
    NSString *sentOrRecieved = isRecieved ? @"received" : @"sent";
    NSString *username = isRecieved ? self.recipient.username : self.me.username;
    
    NSString *javascript = [NSString stringWithFormat: kJsTemplate, sentOrRecieved, messageText, username, jsonDate];
    [self.webView stringByEvaluatingJavaScriptFromString: javascript];
    
    [self.addedMessages setObject: @1 forKey: message.digest];
}

- (CGFloat) cellHeight {
    return self.view.frame.size.height - self.navigationController.navigationBar.frame.size.height - [UIApplication sharedApplication].statusBarFrame.size.height - self.toolbar.frame.size.height;
}
#pragma mark - Helpers

- (NSString *) htmlEncode:(NSString *) str {
    
    return [str stringByReplacingOccurrencesOfString: @"'" withString: @"\\'"];
}

#pragma mark - Webview Delegate

- (void)webViewDidFinishLoad:(UIWebView *)webView {
    isPageLoaded = YES;
    [self loadNewMessages];
}
#pragma mark - TextField Delegate
- (BOOL) textFieldShouldBeginEditing:(UITextField *)textField {
    return YES;
}
- (BOOL) textFieldShouldEndEditing:(UITextField *)textField {
    return YES;
}
- (BOOL)textFieldShouldReturn:(UITextField *)theTextField {
    [self sendTapped];
    return YES;
}

#pragma mark - TableViewDelegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.row == 0) {
        return [self cellHeight];
    }
    
    return self.toolbar.frame.size.height;

}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    cell.separatorInset = UIEdgeInsetsZero;
}


@end
