//
//  UserMessage.swift
//  PrettyGoodChat
//
//  Created by Alex Grinman on 11/17/14.
//  Copyright (c) 2014 alexgrinman. All rights reserved.
//

import Foundation


private let kToKey          = "to"
private let kFromKey        = "from"
private let kMessageKey     = "message"
private let kMessageIdKey   = "mid"
private let kSigKey         = "signature"
private let kDigestKey      = "digest"

@objc public class UserMessage:NSObject {
    
    var to:NSString
    var from:NSString
    var message:ProtocolMessage
    var signature:NSData?
    var digest:NSData
    var messageId:NSNumber?
    
    public init(to:NSString, from:NSString, message: ProtocolMessage, signature:NSData?, digest:NSData?, messageId:NSNumber?) {
        
        self.to         = to
        self.from       = from
        self.message    = message
        self.signature  = signature
        self.digest     = SecKeyWrapper.SHA256(message.serialize()?.dataUsingEncoding(NSUTF8StringEncoding))

        self.messageId  = messageId
    }
    
    public func serialize(dataStore:DataStore, keyStore:SecKeyWrapper, transportable:Bool) -> NSString? {
        
        let parameters:NSDictionary = serializeToDictionary(dataStore, keyStore: keyStore, transportable: transportable)
        
        var err:NSError?
        
        let jsonData = NSJSONSerialization.dataWithJSONObject(parameters, options: NSJSONWritingOptions.PrettyPrinted, error:  &err)
        
        if (err != nil) {
            println(err)
            return nil
        }
        
        return NSString(data: jsonData!, encoding: NSUTF8StringEncoding)
    }
    
    public func serializeToDictionary(dataStore:DataStore, keyStore:SecKeyWrapper, transportable:Bool) -> NSDictionary {
        let parameters = NSMutableDictionary()
        parameters.setObject(self.to, forKey:  kToKey)
        parameters.setObject(self.from, forKey: kFromKey)
        
        if let mid = self.messageId {
            parameters.setObject(mid, forKey: kMessageIdKey)
        }
        
        let message = self.message.serialize()!
        
        //if not transportable, then encrypt with own public key
        var encryptionKey:SecKeyRef
        if (transportable) {
            encryptionKey = dataStore.publicKeyFor(self.to)!
        } else {
            encryptionKey = keyStore.getPublicKeyRef()!.takeUnretainedValue()
        }
        
        
        let encryptedMessage = keyStore.wrapSymmetricKey(message.dataUsingEncoding(NSUTF8StringEncoding)!, keyRef: encryptionKey)
        let signedMessage = keyStore.getSignatureBytes(encryptedMessage!)
        
        parameters.setObject(encryptedMessage!.base64EncodedStringWithOptions(NSDataBase64EncodingOptions.Encoding64CharacterLineLength), forKey: kMessageKey)
        parameters.setObject(signedMessage!.base64EncodedStringWithOptions(NSDataBase64EncodingOptions.Encoding64CharacterLineLength), forKey: kSigKey)
        
        parameters.setObject(digest.base64EncodedStringWithOptions(NSDataBase64EncodingOptions.Encoding64CharacterLineLength), forKey: kDigestKey)

        
        
        return parameters
    }

    
    public class func deserializeString(object: NSString, dataStore:DataStore, keyStore:SecKeyWrapper, transported:Bool) -> UserMessage? {
        let data = object.dataUsingEncoding(NSUTF8StringEncoding)
        
        var err:NSError?
        
        var jsonResult: NSDictionary = NSJSONSerialization.JSONObjectWithData(data!, options: NSJSONReadingOptions.MutableContainers, error: &err) as NSDictionary
        
        if err != nil {
            println(err)
            return nil
        }
        
        return UserMessage.deserializeDictionary(jsonResult, dataStore: dataStore, keyStore: keyStore, transported: transported)
    }
    
    public class func deserializeDictionary(jsonResult: NSDictionary, dataStore:DataStore, keyStore:SecKeyWrapper, transported:Bool) -> UserMessage? {

        
        let to  = jsonResult.objectForKey(kToKey) as NSString?
        let from = jsonResult.objectForKey(kFromKey) as NSString?
        let message = jsonResult.objectForKey(kMessageKey) as NSString?
        let signature = jsonResult.objectForKey(kSigKey) as NSString?
        let messageId = jsonResult.objectForKey(kMessageIdKey) as NSNumber?

        if (to == nil || from == nil || message == nil || signature == nil) {
            println("missing parameters")
            return nil
        }
        
        
        //get digest
        var digest:NSData?
        if let digestB64 = jsonResult.objectForKey(kDigestKey) as NSString? {
            digest = NSData(base64EncodedString: digestB64, options: NSDataBase64DecodingOptions.IgnoreUnknownCharacters)
        }
        
        //verify and decrypt
        let messageData = NSData(base64EncodedString: message!, options: NSDataBase64DecodingOptions.IgnoreUnknownCharacters)
        let signatureData = NSData(base64EncodedString: signature!, options: NSDataBase64DecodingOptions.IgnoreUnknownCharacters)
        
        //if not transported, then stored securely, choose correct signature key
        var signatureKey:SecKeyRef
        if (transported) {
            if let pubKey = dataStore.publicKeyFor(from!) {
                signatureKey = pubKey
            } else {
                return nil
            }
        } else {
            signatureKey = keyStore.getPublicKeyRef()!.takeUnretainedValue()
        }
        
        
        let isSignedCorrectly = keyStore.verifySignature(messageData, secKeyRef: signatureKey, signature: signatureData)
        
        if (isSignedCorrectly == false) {
            println("Signature does not match message!")
            return nil
        }
        
        let decryptedMessageData =  keyStore.unwrapSymmetricKey(messageData!)
        
        if (decryptedMessageData == nil) {
            println("could not decrypt message")
            return nil
        }
        
        let decryptedMessageString = NSString(data: decryptedMessageData, encoding: NSUTF8StringEncoding)!
    
        
        var protocolMessage:ProtocolMessage = ProtocolMessage.deserialize(decryptedMessageString)!
        
        
        return UserMessage(to: NSString.getFingerprintFromPublicKey(keyStore.getPublicKeyBits())!, from: from!, message: protocolMessage, signature: signatureData!, digest: digest, messageId: messageId)
    }
}